package com.chelvc.framework.database.handler;

import java.lang.reflect.Type;
import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import com.chelvc.framework.base.context.JacksonContextHolder;
import com.chelvc.framework.common.util.ObjectUtils;
import lombok.NonNull;
import org.apache.ibatis.executor.result.ResultMapException;
import org.apache.ibatis.type.JdbcType;
import org.apache.ibatis.type.TypeException;
import org.apache.ibatis.type.TypeHandler;
import org.apache.ibatis.type.TypeReference;

/**
 * JSON类型处理器接口
 *
 * @param <T> 对象类型
 * @author Woody
 * @date 2024/1/30
 */
public interface JsonTypeHandler<T> extends TypeHandler<T> {
    /**
     * 获取对象类型
     *
     * @return 对象类型
     */
    Type getRawType();

    @Override
    default void setParameter(PreparedStatement ps, int i, T parameter, JdbcType jdbcType) throws SQLException {
        if (parameter == null) {
            if (jdbcType == null) {
                throw new TypeException("JDBC requires that the JdbcType must be specified " +
                        "for all nullable parameters.");
            }
            try {
                ps.setNull(i, jdbcType.TYPE_CODE);
            } catch (SQLException e) {
                throw new TypeException("Error setting null for parameter #" + i + " with JdbcType " +
                        jdbcType + " . Try setting a different JdbcType for this parameter " +
                        "or a different jdbcTypeForNull configuration property. Cause: " + e, e);
            }
        } else {
            try {
                setNonNullParameter(ps, i, parameter, jdbcType);
            } catch (Exception e) {
                throw new TypeException("Error setting non null for parameter #" + i + " with JdbcType " +
                        jdbcType + " . Try setting a different JdbcType for this parameter " +
                        "or a different configuration property. Cause: " + e, e);
            }
        }
    }

    @Override
    default T getResult(ResultSet rs, String columnName) throws SQLException {
        try {
            return getNullableResult(rs, columnName);
        } catch (Exception e) {
            throw new ResultMapException("Error attempting to get column '" + columnName + "' from result set.  " +
                    "Cause: " + e, e);
        }
    }

    @Override
    default T getResult(ResultSet rs, int columnIndex) throws SQLException {
        try {
            return getNullableResult(rs, columnIndex);
        } catch (Exception e) {
            throw new ResultMapException("Error attempting to get column #" + columnIndex + " from result set.  " +
                    "Cause: " + e, e);
        }
    }

    @Override
    default T getResult(CallableStatement cs, int columnIndex) throws SQLException {
        try {
            return getNullableResult(cs, columnIndex);
        } catch (Exception e) {
            throw new ResultMapException("Error attempting to get column #" + columnIndex + " from callable statement" +
                    ".  Cause: " + e, e);
        }
    }

    /**
     * 设置参数值
     *
     * @param ps        PreparedStatement对象实例
     * @param i         参数下标
     * @param parameter 参数值
     * @param jdbcType  数据库字段类型
     * @throws SQLException SQL异常
     */
    default void setNonNullParameter(PreparedStatement ps, int i, T parameter, JdbcType jdbcType) throws SQLException {
        if (ObjectUtils.isBlank(parameter)) {
            ps.setNull(i, Types.VARCHAR);
        } else {
            ps.setString(i, JacksonContextHolder.serialize(parameter));
        }
    }

    /**
     * 获取字段结果
     *
     * @param rs         结果集合对象实例
     * @param columnName 字段名称
     * @return 字段值
     * @throws SQLException SQL异常
     */
    @SuppressWarnings("unchecked")
    default T getNullableResult(ResultSet rs, String columnName) throws SQLException {
        return (T) JacksonContextHolder.deserialize(rs.getString(columnName), this.getRawType());
    }

    /**
     * 获取字段结果
     *
     * @param rs          结果集合对象实例
     * @param columnIndex 字段下标
     * @return 字段值
     * @throws SQLException SQL异常
     */
    @SuppressWarnings("unchecked")
    default T getNullableResult(ResultSet rs, int columnIndex) throws SQLException {
        return (T) JacksonContextHolder.deserialize(rs.getString(columnIndex), this.getRawType());
    }

    /**
     * 获取字段结果
     *
     * @param cs          CallableStatement对象实例
     * @param columnIndex 字段下标
     * @return 字段值
     * @throws SQLException SQL异常
     */
    @SuppressWarnings("unchecked")
    default T getNullableResult(CallableStatement cs, int columnIndex) throws SQLException {
        return (T) JacksonContextHolder.deserialize(cs.getString(columnIndex), this.getRawType());
    }

    /**
     * JSON类型处理器默认实现
     *
     * @param <T> 对象类型
     */
    class Default<T> extends TypeReference<T> implements JsonTypeHandler<T> {
    }

    /**
     * JSON类型处理器简单实现
     *
     * @param <T> 对象类型
     */
    class Simple<T> implements JsonTypeHandler<T> {
        private final Type type;

        public Simple(@NonNull Type type) {
            this.type = type;
        }

        @Override
        public Type getRawType() {
            return this.type;
        }
    }
}
