package com.chelvc.framework.database.support;

import java.util.Collection;
import java.util.List;
import java.util.function.Function;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.IService;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import lombok.NonNull;

/**
 * 业务增强操作接口
 *
 * @param <T> 数据模型类型
 * @author Woody
 * @date 2024/3/10
 */
public interface EnhanceService<T> extends IService<T> {
    /**
     * 创建或忽略实体
     *
     * @param entity 实体对象实例
     * @return true/false
     */
    default boolean createIgnore(@NonNull T entity) {
        return ((EnhanceMapper<T>) this.getBaseMapper()).insertIgnore(entity) > 0;
    }

    /**
     * 创建或修改实体
     *
     * @param entity 实体对象实例
     * @return true/false
     */
    default boolean createUpdate(@NonNull T entity) {
        return ((EnhanceMapper<T>) this.getBaseMapper()).insertUpdate(entity) > 0;
    }

    /**
     * 批量创建或忽略实体
     *
     * @param entities 实体对象实例集合
     * @return true/false
     */
    default boolean batchCreateIgnore(@NonNull Collection<T> entities) {
        return !entities.isEmpty() && ((EnhanceMapper<T>) this.getBaseMapper()).batchInsertIgnore(entities) > 0;
    }

    /**
     * 批量创建或修改实体
     *
     * @param entities 实体对象实例集合
     * @return true/false
     */
    default boolean batchCreateUpdate(@NonNull Collection<T> entities) {
        return !entities.isEmpty() && ((EnhanceMapper<T>) this.getBaseMapper()).batchInsertUpdate(entities) > 0;
    }

    @Override
    default EnhanceQueryWrapper<T> query() {
        return new EnhanceQueryWrapper<>(this.getBaseMapper());
    }

    @Override
    default EnhanceLambdaQueryWrapper<T> lambdaQuery() {
        return new EnhanceLambdaQueryWrapper<>(this.getBaseMapper());
    }

    /**
     * 分页查询
     *
     * @param paging 查询分页信息
     * @return 数据分页信息
     */
    default Pagination<T> paging(@NonNull Paging paging) {
        return this.paging(Wrappers.emptyWrapper(), paging);
    }

    /**
     * 分页查询
     *
     * @param paging  查询分页信息
     * @param adapter 数据转换适配器
     * @param <R>     目标数据类型
     * @return 数据分页信息
     */
    default <R> Pagination<R> paging(@NonNull Paging paging, @NonNull Function<List<T>, @NonNull List<R>> adapter) {
        return this.paging(Wrappers.emptyWrapper(), paging, adapter);
    }

    /**
     * 分页查询
     *
     * @param query  查询对象
     * @param paging 查询分页信息
     * @return 数据分页信息
     */
    default Pagination<T> paging(@NonNull Wrapper<T> query, @NonNull Paging paging) {
        return DatabaseContextHolder.paging(paging, page -> this.page(page, query));
    }

    /**
     * 分页查询
     *
     * @param query   查询对象
     * @param paging  查询分页信息
     * @param adapter 数据转换适配器
     * @param <R>     目标数据类型
     * @return 数据分页信息
     */
    default <R> Pagination<R> paging(@NonNull Wrapper<T> query, @NonNull Paging paging,
                                     @NonNull Function<List<T>, List<R>> adapter) {
        return DatabaseContextHolder.paging(paging, page -> this.page(page, query), adapter);
    }
}
