package com.chelvc.framework.location.support;

import java.time.Duration;
import java.util.concurrent.ThreadLocalRandom;
import java.util.function.Supplier;

import com.chelvc.framework.location.Address;
import com.chelvc.framework.location.LocationHandler;
import com.chelvc.framework.redis.context.RedisContextHolder;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 可缓存的地址定位实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class CacheableLocationHandler implements LocationHandler {
    private final int expiration;
    private final LocationHandler handler;

    public CacheableLocationHandler(@NonNull LocationHandler handler, int expiration) {
        this.handler = handler;
        this.expiration = expiration;
    }

    /**
     * 执行地址定位解析
     *
     * @param code     地址标识
     * @param supplier 地址解析回调函数
     * @return 地址信息
     */
    private Address execute(String code, Supplier<Address> supplier) {
        String key = "location:" + code;
        Address address = null;
        try {
            address = (Address) RedisContextHolder.getDefaultTemplate().opsForValue().get(key);
        } catch (Exception e) {
            log.warn("Redis location address get failed: {}", e.getMessage());
        }
        if (address == null && (address = supplier.get()) != null) {
            // 为避免缓存雪崩，缓存过期时间增加随机数
            int random = ThreadLocalRandom.current().nextInt(60);
            Duration duration = Duration.ofSeconds(this.expiration + random);
            try {
                RedisContextHolder.getDefaultTemplate().opsForValue().set(key, address, duration);
            } catch (Exception e) {
                log.warn("Redis location address save failed: {}", e.getMessage());
            }
        }
        return address;
    }

    @Override
    public Address ip2address(@NonNull String ip) {
        return this.execute(ip, () -> this.handler.ip2address(ip));
    }

    @Override
    public Address mobile2address(@NonNull String mobile) {
        return this.execute(mobile, () -> this.handler.mobile2address(mobile));
    }

    @Override
    public Address location2address(@NonNull String location) {
        return this.execute(location, () -> this.handler.location2address(location));
    }

    @Override
    public Address location2address(double longitude, double latitude) {
        String location = longitude + "," + latitude;
        return this.execute(location, () -> this.handler.location2address(longitude, latitude));
    }
}
